from Globals import DTMLFile
from OFS.Folder import Folder
from zope.security.interfaces import Unauthorized

from cgi import FieldStorage

from trac.web.main import dispatch_request, send_project_index
from trac.web.wsgi import WSGIGateway
from trac.web.api import Request

from trac.perm import PermissionError

manage_addZopeTracForm = DTMLFile('dtml/manage_addZopeTracForm', globals())

from logging import getLogger

LOG = getLogger('ZopeTrac')


def manage_addZopeTrac(self, id, tracEnvParentDir, title = '', REQUEST = {}):
	""" Add a Zope Trac Instance """
	o = ZopeTrac(id, tracEnvParentDir, title)
	self._setObject(id, o)

	if REQUEST:
		return self.manage_main(self, REQUEST)
	return ''

class TracFieldStorage:
	def __init__(self, formStuff):
		self.formStuff = formStuff
		for k, v in self.formStuff.items():
			if not isinstance(v, str):
				print dir(v)
				v.file = v

	def __getitem__(self, key):
		return self.formStuff[key]

	def get(self, key, default = None):
		return self.formStuff.get(key, default)

	def getvalue(self, key, default = None):
		return self.formStuff.get(key, default)

	def getlist(self, key):
		x = self.formStuff.get(key, [])
		if not isinstance(x, list):
			x = [x,]
		return x

	def keys(self):
		return self.formStuff.keys()

	def has_key(self, key):
		return self.formStuff.has_key(key)

	def __len__(self):
		return len(self.formStuff.keys())

	def __setitem__(self, key, value):
		self.formStuff[key] = value

class ZopeGateway(WSGIGateway):
	""" A gateway to Trac """

	
	def __init__(self, REQUEST):
		self.environ = REQUEST.environ
		WSGIGateway.__init__(self, REQUEST.environ)
		self.REQUEST = REQUEST
		Request._parse_args = self._parse_args

	def _sendfile(self, fileObj):
		self.REQUEST.RESPONSE.write(fileObj.read())

	def _write(self, data):
		assert self.headers_set, 'Response not started'

		if not self.headers_sent:
			status, headers = self.headers_set = self.headers_set
			self.REQUEST.RESPONSE.setStatus(status)
			for header in headers:
				if header[0] == 'Location':
					self.REQUEST.RESPONSE.redirect(header[1])
				elif header[0] == 'Set-Cookie':
					cookies = header[1].strip().split(';')
					for cookie in cookies:
						if not cookie:
							continue
						c, v = cookie.strip().split('=')
						self.REQUEST.RESPONSE.setCookie(c, v[1:-1])
				else:
					self.REQUEST.RESPONSE.setHeader(*header)
		self.REQUEST.RESPONSE.write(data)

	def _parse_args(self):
		return TracFieldStorage(self.REQUEST.form)

class ZopeTrac(Folder):
	""" Map a Trac Instance to a Zope Folder """

	def __init__(self, id, tracEnvParentDir = '', title = ''):
		self.id = id
		self.title = title
		self.tracEnvParentDir = tracEnvParentDir

	def __call__(self, *args, **kwargs):
		# Tell Trac where our trac is.
		e = self.REQUEST.environ
		
		e['trac.env_path'] = self.tracEnvParentDir

		# Re-jigger the PATH_INFO so trac knows what URL we want.
		baseUrl = '/' + self.virtual_url_path()
		path_info = e['PATH_TRANSLATED'].split('VirtualHostRoot')[-1]
		newUrl = path_info[len(baseUrl):]
		if not newUrl:
			newUrl = '/'
		e['PATH_INFO'] = newUrl
		e['trac.base_url'] = self.absolute_url() + '/'
		e['trac.abs_href'] = self.absolute_url() + '/'
		e['trac.base_path'] = newUrl
		
		e['SCRIPT_NAME'] = self.absolute_url() + '/'

		e['REMOTE_ADDR'] = self.REQUEST.REMOTE_ADDR
		e['REMOTE_USER'] = self.REQUEST.AUTHENTICATED_USER.getUserName()
		e['SERVER_NAME'] = self.REQUEST.SERVER_NAME
		e['SERVER_PORT'] = self.REQUEST.SERVER_PORT

		# Start our gateway session
		gw = ZopeGateway(self.REQUEST)
		gw.run(dispatch_request)

	def __bobo_traverse__(self, REQUEST, name):
		return self
